extern "C"
{
#include "lua.h"
#include "lualib.h"
#include "lauxlib.h"
}

const char luaScript[] = 
"dog = { speed = 10, x = 1, y = 1 }            \n"
"cat = { speed = 5, x = 10, y = 10 }           \n"
"                                              \n"
"function chase(self, target)                  \n"
"	local dx, dy, distance, time               \n"
"	dx = target.x - self.x                     \n"
"	dy = target.y - self.y                     \n"
"	distance = math.sqrt(dx*dx+dy*dy)          \n"
"	while distance > 1 do                      \n"
"		time = coroutine.yield(self.x, self.y) \n"
"		dx = dx/distance                       \n"
"		dy = dx/distance                       \n"
"		self.x = self.x + dx*time*self.speed   \n"
"		self.y = self.y + dy*time*self.speed   \n"
"		dx = target.x - self.x                 \n"
"		dy = target.y - self.y                 \n"
"		distance = math.sqrt(dx*dx+dy*dy)      \n"
"	end                                        \n"
"	print 'Chomp!'                             \n"
"	return self.x, self.y                      \n"
"end                                           \n"
"                                              \n"
"function flee(self, target)                   \n"
"	local dx, dy, distance, time               \n"
"	dx = self.x - target.x                     \n"
"	dy = self.y - target.y                     \n"
"	distance = math.sqrt(dx*dx+dy*dy)          \n"
"	while distance > 1 do                      \n"
"		time = coroutine.yield(self.x, self.y) \n"
"		dx = dx/distance + 0.5-math.random()   \n"
"		dy = dx/distance + 0.5-math.random()   \n"
"		self.x = self.x + dx*time*self.speed   \n"
"		self.y = self.y + dy*time*self.speed   \n"
"		dx = self.x - target.x                 \n"
"		dy = self.y - target.y                 \n"
"		distance = math.sqrt(dx*dx+dy*dy)      \n"
"	end                                        \n"
"	print \"I'm dead!\"                        \n"
"	return self.x, self.y                      \n"
"end                                           \n";


int main(int argc, char *argv[])
{
	lua_State *luaState, *dogThread, *catThread;
	int result, dogDone = 0, catDone = 0;
	lua_Number dogX, dogY, catX, catY;
	
	luaState = lua_open();
	luaopen_base(luaState);
	luaopen_math(luaState);

	/* Compile the script */
	if((result = luaL_loadbuffer(luaState, luaScript, sizeof(luaScript)-1, "luaScript")) != 0)
	{
		fprintf(stderr, lua_tostring(luaState, -1));
		return result;
	}
	/* Run the script to create the tables and the functions */
	if((result = lua_pcall(luaState, 0, 2, 0)) != 0)
	{
		fprintf(stderr, lua_tostring(luaState, -1));
		return result;
	}

	/* Make a new thread for the dog */
	dogThread = lua_newthread(luaState);
	/* Store the thread in a variable for later reference */
	lua_setglobal(luaState, "dogThread");
	/* Make a coroutine out of the chase() function in the dog thread */
	lua_getglobal(dogThread, "chase");
	lua_getglobal(dogThread, "dog");
	lua_getglobal(dogThread, "cat");
	if((result = lua_resume(dogThread, 2)) != 0)
	{
		printf(lua_tostring(dogThread, -1));
		return result;
	}
	dogX = lua_tonumber(dogThread, -2);
	dogY = lua_tonumber(dogThread, -1);
	lua_pop(dogThread, 2);

	/* Make a new thread for the cat */
	catThread = lua_newthread(luaState);
	/* Store the thread in a variable for later reference */
	lua_setglobal(luaState, "catThread");
	/* Make a coroutine out of the flee() function in the cat thread */
	lua_getglobal(catThread, "flee");
	lua_getglobal(catThread, "cat");
	lua_getglobal(catThread, "dog");
	if((result = lua_resume(catThread, 2)) != 0)
	{
		printf(lua_tostring(catThread, -1));
		return result;
	}
	catX = lua_tonumber(catThread, -2);
	catY = lua_tonumber(catThread, -1);
	lua_pop(catThread, 2);

	while(!(dogDone || catDone))
	{
		printf("dog: (%f, %f) cat:(%f, %f)\n", dogX, dogY, catX, catY);

		if (!dogDone)
		{	/* Update the dog */
			lua_pushnumber(dogThread, 0.1); /* time value returned from yield() */
			if((result = lua_resume(dogThread, 1)) != 0)
			{
				printf(lua_tostring(dogThread, -1));
				dogDone = 1;
			}
			else
			{	/* Get the x,y values passed as parameters to yeild() */
				dogX = lua_tonumber(dogThread, -2);
				dogY = lua_tonumber(dogThread, -1);
				lua_pop(dogThread, 2);
			}
		}
		if (!catDone)
		{	/* Update the cat */
			lua_pushnumber(catThread, 0.1); /* time value returned from yield() */
			if((result = lua_resume(catThread, 1)) != 0)
			{
				printf(lua_tostring(catThread, -1));
				catDone = 1;
			}
			else
			{	/* Get the x,y values passed as parameters to yeild() */
				catX = lua_tonumber(catThread, -2);
				catY = lua_tonumber(catThread, -1);
				lua_pop(catThread, 2);
			}
		}
	}

	lua_close(luaState);
	return 0;
}