program cotest;

{$mode objfpc}{$H+}

uses
  {$IFDEF UNIX}{$IFDEF UseCThreads}
  cthreads,
  {$ENDIF}{$ENDIF}
  Classes, LuaWrapper, lua, lualib, lauxlib, LuaUtils,
  sysutils;

var
  RootScript : TLUA;
  dogThread : PLua_State;
  catThread : PLua_State;
  dogDone,
  catDone   : Boolean;
  rres      : Integer;
  dogX, dogY,
  catX, catY : lua_Number;

function luaPrint(L:PLua_State) : Integer; cdecl;
var
  i, n : Integer;
begin
  n := lua_gettop(L);
  for i := 1 to n do
    write(LuaToVariant(L, i));
end;

function luaResume(L : PLua_State; NArgs:Integer; var Res : Integer) : Boolean;
begin
  Res := lua_resume(L, NArgs);
  result := Res <> 0;
end;

begin
  RootScript := TLUA.Create(nil);
  try
    // Make sure that you load the script before you register the methods
    //RootScript.LoadScript('print("Test\n"); for i = 1,10 do print(i.."\n") end;');
    RootScript.LoadFile('script.lua');
    // Now we register the methods
    RootScript.RegisterLUAMethod('print', @luaPrint);
    // Execute the script
    RootScript.Execute;
    
    dogDone := false;
    catDone := false;
    
    //create a thread for the dog
    dogThread := lua_newthread(RootScript.LuaState);
    //Store a pointer to the thread for later use
    lua_setglobal(RootScript.LuaState, 'dogThread');
    //Get dog.process method
    lua_pushstring(dogThread, 'dog');
    lua_gettable(dogThread, LUA_GLOBALSINDEX);
    luaPushString(dogThread, 'process');
    lua_rawget(dogThread, -2);
    //Push the params onto the stack
    lua_getglobal(dogThread, 'dog');
    lua_getglobal(dogThread, 'cat');
    //Call dog.process
    if luaResume(dogThread, 2, rres) then
      begin
        writeln(lua_tostring(dogThread, -1));
        writeln(#13#10'Press any key to exit...');
        readln();
        exit;
      end;
    dogX := lua_tonumber(dogThread, -2);
    dogY := lua_tonumber(dogThread, -1);
    lua_pop(dogThread, 2);
    
    //create a thread for the cat
    catThread := lua_newthread(RootScript.LuaState);
    //Store a pointer to the thread for later use
    lua_setglobal(RootScript.LuaState, 'catThread');
    //get cat.process method
    lua_pushstring(catThread, 'cat');
    lua_gettable(catThread, LUA_GLOBALSINDEX);
    luaPushString(catThread, 'process');
    lua_rawget(catThread, -2);
    //Push the params onto the stack
    lua_getglobal(catThread, 'cat');
    lua_getglobal(catThread, 'dog');
    //Call dog.process
    if luaResume(catThread, 2, rres) then
      begin
        writeln(lua_tostring(catThread, -1));
        writeln(#13#10'Press any key to exit...');
        readln();
        exit;
      end;
    catX := lua_tonumber(catThread, -2);
    catY := lua_tonumber(catThread, -1);
    lua_pop(catThread, 2);

    while not(dogDone or catDone) do
      begin
        writeln(format('dog: (%f, %f) cat:(%f, %f)', [dogX, dogY, catX, catY]));
        if not dogDone then
          begin
            lua_pushnumber(dogThread, 0.1);
            if luaResume(dogThread, 1, rres) then
              begin
                writeln(lua_tostring(dogThread, -1));
                dogDone := true;
              end
            else
              begin
                dogX := lua_tonumber(dogThread, -2);
                dogY := lua_tonumber(dogThread, -1);
                lua_pop(dogThread, 2);
              end;
          end;
        if not catDone then
          begin
            lua_pushnumber(catThread, 0.1);
            if luaResume(catThread, 1, rres) then
              begin
                writeln(lua_tostring(catThread, -1));
                catDone := true;
              end
            else
              begin
                catX := lua_tonumber(catThread, -2);
                catY := lua_tonumber(catThread, -1);
                lua_pop(catThread, 2);
              end;
          end;
      end;

    // Read a line, good for a pause
    writeln(#13#10'Done'#13#10'Press any key to exit...');
    readln();
  finally
    RootScript.Free; // this performs the lua_close
  end;
end.

